<?php

declare(strict_types=1);

namespace yckj\middleware;

use Closure;
use think\Config;
use think\Request;
use think\Response;
use think\Facade\Log;

/**
 * 跨域中间件
 * Class AllowCrossDomain
 * 
 * @package \yckj\commons\middleware
 */
class AllowCrossDomain
{

    protected $cookieDomain;

    // header头配置
    protected $header = [
        "Access-Control-Allow-Origin" => "*",
        'Access-Control-Max-Age' => 1800,
        'Access-Control-Allow-Methods' => 'GET, POST, PATCH, PUT, DELETE, OPTIONS',
        'Access-Control-Allow-Headers' => 'Authorization, Content-Type, If-Match, If-Modified-Since, If-None-Match, If-Unmodified-Since, X-CSRF-TOKEN, X-Requested-With,content-type,ignore,token', //如果有新增header字段，在这里添加
    ];


    /**
     * AllowCrossDomain constructor.
     * @param Config $config
     */
    public function __construct(Config $config)
    {
        $this->cookieDomain = $config->get('cookie.domain', '');
    }

    /**
     * 允许跨域请求
     * 
     * @access public
     * @param Request $request
     * @param Closure $next
     * @param array $header
     * @return Response
     */
    public function handle(Request $request, Closure $next, ?array $header = [])
    {
        Log::debug(__NAMESPACE__ . ': Changing Header: ' . implode(' ', $header));
        
        // 已有 header 则合并
        $header = !empty($header) ? array_merge($this->header, $header) : $this->header;

        // 调整跨域限制
        if (!isset($header['Access-Control-Allow-Origin'])) {
            $origin = $request->header('origin');

            if ($origin && ('' == $this->cookieDomain || strpos($origin, $this->cookieDomain))) {
                $header['Access-Control-Allow-Origin'] = $origin;
            } else {
                $header['Access-Control-Allow-Origin'] = '*';
            }
        }
        Log::debug(__NAMESPACE__ . ': Header Changed: ' . implode(' ', $header));

        return $next($request)->header($header);
    }
}
